@@ -1,4 +1,4 @@
-/* $Id: xfce-rc-simple.c,v 1.13 2004/11/18 23:27:57 benny Exp $ */
+/* $Id: xfce-rc-simple.c 20575 2006-03-26 10:02:32Z benny $ */
 /*-
  * Copyright (c) 2003-2004 Benedikt Meurer <benny@xfce.org>
  * All rights reserved.
@@ -179,7 +179,28 @@ simple_add_entry (XfceRcSimple *simple,
 
   if (G_UNLIKELY (entry == NULL))
     {
-      g_return_val_if_fail (locale == NULL, NULL);
+      if (locale != NULL) {
+        /* this point is reached when there is a .desktop file that lists a language specific key for something that had no language-neutral key yet.
+           Example:
+             b.desktop
+             
+             [Desktop Entry]
+             Version=1.0
+             Encoding=UTF-8
+             Name=xyz
+             GenericName[de_AT]=Test
+             
+           here GenericName[de_AT] will end up here.
+           The previous way with g_return_val_if_fail would call an assertion failure and terminate the _whole application_(!!).
+           
+           Saner ways to react are either just ignoring GenericName[de_AT] altogether, or, alternatively, just set the normal GenericName
+           to Test too (i.e. imply GenericName=Test).
+           
+           For now, we are just ignoring the line altogether. But we aren't assert-failing anymore and the apps dont crash anymore.
+           */
+        return NULL;
+      }
+      /* why you annoying macro, will you stop borking libxfceutil? thanks. DO NOT DO THAT: g_return_val_if_fail (locale == NULL, NULL); */
 
       entry         = g_chunk_new (Entry, simple->entry_chunk);
       entry->key    = g_string_chunk_insert (simple->string_chunk, key);
@@ -354,19 +375,24 @@ simple_parse_line (gchar  *line,
 
       q = r + strlen (r);
 
-      while (q > r && (g_ascii_isspace (*(q-1)) || ((*(q-1)) == '\r')))
+      /* "\ " at the end of the string will not be removed */
+      while (q > r && ((g_ascii_isspace (*(q-1)) && *(q-2) != '\\') || ((*(q-1)) == '\r')))
         --q;
 
       *value = r;
       *q = '\0';
 
-      /* unescape \n, \t, \r and \\ */
+      /* unescape \ , \n, \t, \r and \\ */
       for (p = r; *r != '\0'; )
         {
           if (G_UNLIKELY (*r == '\\'))
             {
               switch (*(r+1))
                 {
+                case ' ':
+                  *p++ = ' ';
+                  break;
+
                 case 'n':
                   *p++ = '\n';
                   break;
@@ -406,11 +432,36 @@ simple_parse_line (gchar  *line,
 static gchar*
 simple_escape (gchar *buffer, gsize size, const gchar *string)
 {
-  gchar *p;
+  const gchar *s;
+  gchar       *p;
 
-  for (p = buffer; p - buffer < size - 2 && *string != '\0'; ++string)
+  /* escape all whitespace at the beginning of the string */
+  for (p = buffer; p - buffer < size - 2 && *string == ' '; ++string)
+    {
+      *p++ = '\\';
+      *p++ = ' ';
+    }
+
+  for (; p - buffer < size - 2 && *string != '\0'; ++string)
     switch (*string)
       {
+      case ' ':
+        /* check if any non whitespace characters follow */
+        for (s = string + 1; g_ascii_isspace (*s); ++s)
+          ;
+        if (*s == '\0')
+          {
+            /* need to escape the space */
+            *p++ = '\\';
+            *p++ = ' ';
+          }
+        else
+          {
+            /* still non-whitespace, no need to escape */
+            *p++ = ' ';
+          }
+        break;
+
       case '\n':
         *p++ = '\\';
         *p++ = 'n';
